/****************************************************************************
 * @file     main.c
 * @version  V1.00
 * @brief    Example to load function to SRAM and call it.
 * @note
 * Copyright (C) 2016 Nuvoton Technology Corp. All rights reserved.
 ****************************************************************************/

#include <stdio.h>
#include "M451Series.h"
#include "func_in_SRAM.h"


#ifdef BUILD_FUNC

/*** Compile the real function and get the binary code ***/

int32_t FuncEntry(int32_t x, int32_t y);

/* The declaration should be kept to avoid function be removed while linking */
PFUNC pFuncAddr = FuncEntry;

#else

/*** Function binary put in an array then call it ***/

uint8_t FuncBin[FUNC_SIZE] = {
    // cut and paste the function binary code in ASCII mode here
    0xAF,0xF3,0x00,0x80,0x88,0x42,0x00,0xDA,
    0x08,0x46,0x70,0x47
};

PFUNC pFuncAddr = (PFUNC)FUNC_ADDR;

#endif


void SYS_Init(void)
{
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init System Clock                                                                                       */
    /*---------------------------------------------------------------------------------------------------------*/

    /* Enable UART module clock */
    CLK_EnableModuleClock(UART0_MODULE);

    /* Update System Core Clock */
    /* User can use SystemCoreClockUpdate() to calculate SystemCoreClock and cyclesPerUs automatically. */
    SystemCoreClockUpdate();

    /*---------------------------------------------------------------------------------------------------------*/
    /* Init I/O Multi-function                                                                                 */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Set PD multi-function pins for UART0 RXD and TXD */
    SYS->GPD_MFPL &= ~(SYS_GPD_MFPL_PD0MFP_Msk | SYS_GPD_MFPL_PD1MFP_Msk);
    SYS->GPD_MFPL |= (SYS_GPD_MFPL_PD0MFP_UART0_RXD | SYS_GPD_MFPL_PD1MFP_UART0_TXD);
}


int main()
{
    uint32_t i;
    uint32_t u32Addr;

    /* Unlock protected registers */
    SYS_UnlockReg();

    SYS_Init();

    /* Lock protected registers */
    SYS_LockReg();

    /* Init UART0 to 115200-8n1 for print message */
    UART_Open(UART0, 115200);

    printf("\nSimple Demo Code\n\n");

    printf("Give 100 and 200.\n");

#ifdef BUILD_FUNC

    /* Dump the function binary code which put in SRAM to console */
    
    u32Addr = (uint32_t)pFuncAddr & 0xFFFFFFFE;

    printf("\n--- Dump Begin ---\n");
    for(i=0; i<FUNC_SIZE-1;) {
        printf("0x%02X,", ((uint8_t *)u32Addr)[i]);

        if(++i%8 == 0)
            printf("\n");
    }
    printf("0x%02X", ((uint8_t *)u32Addr)[i]);
    printf("\n--- Dump End ---\n");

#else

    u32Addr = (uint32_t)pFuncAddr & 0xFFFFFFFE;

    /*
        Start to fill the function binary to SRAM, then call it.
        You can modify here to load the function binary from other interface
        and fill to SRAM.
    */

    printf("Fill the binary code to SRAM...\n");
    for(i=0; i<FUNC_SIZE; i++)
        ((uint8_t *)u32Addr)[i] = FuncBin[i];

    printf("Call the function. And ");
    printf("Get %d.\n", (*pFuncAddr)(100, 200));

#endif

    while(1);
}

/*** (C) COPYRIGHT 2016 Nuvoton Technology Corp. ***/
